@@ -1,5 +1,5 @@
 import sys
-from xmlrpc.client import Marshaller
+import re
 
 if __name__ == "__main__":
 
@@ -13,7 +13,17 @@ from q2db.schema import Q2DbSchema
 from q2db.db import Q2Db
 from q2rad.q2actions import Q2Actions
 from q2db.cursor import Q2Cursor
-from q2rad.q2raddb import read_url, open_url
+from q2rad.q2raddb import (
+    read_url,
+    open_url,
+    insert,
+    raw_insert,
+    update,
+    delete,
+    transaction,
+    commit,
+    rollback,
+)
 
 from q2gui import q2app
 from q2rad.q2raddb import q2cursor
@@ -55,12 +65,13 @@ def get_report(report_name):
 
 
 def run_module(module_name):
-    q2app.q2_app.run_module(module_name)
+    return q2app.q2_app.run_module(module_name)
 
 
 class Q2RadApp(Q2App):
     def __init__(self, title=""):
         super().__init__(title)
+        self.settings_title = "q2RAD"
         self.db = None
 
         self.db_data = None
@@ -71,19 +82,13 @@ class Q2RadApp(Q2App):
 
         self.q2market_path = "../q2market"
 
-        self.q2market_url = (
-            "https://raw.githubusercontent.com/AndreiPuchko/q2market/main/"
-        )
+        self.q2market_url = "https://raw.githubusercontent.com/AndreiPuchko/q2market/main/"
 
-        self.assets_url = (
-            "https://raw.githubusercontent.com/AndreiPuchko/q2gui/main/assets/"
-        )
+        self.assets_url = "https://raw.githubusercontent.com/AndreiPuchko/q2gui/main/assets/"
 
         qss_file = "q2gui.qss"
         if not os.path.isfile(qss_file):
-            qss_url = (
-                "https://raw.githubusercontent.com/AndreiPuchko/q2rad/main/q2gui.qss"
-            )
+            qss_url = "https://raw.githubusercontent.com/AndreiPuchko/q2rad/main/q2gui.qss"
             open(qss_file, "w").write(read_url(qss_url).decode("utf-8"))
 
         if os.path.isfile(qss_file):
@@ -127,12 +132,12 @@ class Q2RadApp(Q2App):
     def open_application(self, autoload_enabled=False):
         Q2AppSelect().run(autoload_enabled)
         if self.selected_application != {}:
-            self.open_selected_app()
+            self.open_selected_app(True)
             self.check_app_update()
         else:
             self.close()
 
-    def open_selected_app(self):
+    def open_selected_app(self, go_to_q2market=False):
         self.clear_app_info()
         self.migrate_db_logic()
         self.migrate_db_data()
@@ -144,7 +149,7 @@ class Q2RadApp(Q2App):
         # self.run_modules()
         # self.run_reports()
         # self.run_app_manager()
-        if (
+        if go_to_q2market and (
             max(
                 [
                     self.db_logic.table("forms").row_count(),
@@ -157,10 +162,7 @@ class Q2RadApp(Q2App):
             )
             <= 0
         ):
-            if (
-                q2AskYN("Application is empty! Would you like to download some App?")
-                == 2
-            ):
+            if q2AskYN("Application is empty! Would you like to download some App?") == 2:
                 Q2Market().run()
 
     def migrate_db_data(self):
@@ -224,13 +226,13 @@ class Q2RadApp(Q2App):
 
         self.create_form_menu()
 
-        self.dev_mode = self.selected_application.get("dev_mode")
+        self.dev_mode = self.selected_application.get("dev_mode") or os.path.isdir(self.q2market_path)
 
         if self.dev_mode:
-            self.add_menu("Dev|Forms", self.run_forms, toolbar=self.dev_mode)
-            self.add_menu("Dev|Modules", self.run_modules, toolbar=self.dev_mode)
-            self.add_menu("Dev|Querys", self.run_queries, toolbar=self.dev_mode)
-            self.add_menu("Dev|Reports", self.run_reports, toolbar=self.dev_mode)
+            self.add_menu("Dev|Forms", self.run_forms, toolbar=self.dev_mode and 0)
+            self.add_menu("Dev|Modules", self.run_modules, toolbar=self.dev_mode and 0)
+            self.add_menu("Dev|Querys", self.run_queries, toolbar=self.dev_mode and 0)
+            self.add_menu("Dev|Reports", self.run_reports, toolbar=self.dev_mode and 0)
         self.build_menu()
         # self.show_toolbar(False)
 
@@ -254,10 +256,15 @@ class Q2RadApp(Q2App):
         for package in q2_modules:
             w.step()
             latest_version, current_version = self.get_package_versions(package)
-            about.append(
-                f"<b>{package}</b>: {current_version}"
-                f"{'(' + latest_version + ' avaiable)' if current_version!=latest_version else ''}"
-            )
+            if latest_version:
+                if current_version != latest_version:
+                    latest_version_text = f"({latest_version })"
+                else:
+                    latest_version_text = ""
+            else:
+                latest_version_text = _(" (Can't to load packacke info)")
+
+            about.append(f"<b>{package}</b>: {current_version}{latest_version_text}")
         w.close()
         q2Mess("<br>".join(about))
 
@@ -265,13 +272,18 @@ class Q2RadApp(Q2App):
         asset_url = f"{self.assets_url}/{name}"
         try:
             asset_content = read_url(asset_url)
+            if asset_content is None:
+                return False
         except Exception:
             print(f"Error reading {asset_url}")
-            return
+            return False
+
         try:
             open(f"assets/{name}", "wb").write(asset_content)
+            return True
         except Exception:
             print(f"Error writing asset/{name}")
+            return False
 
     def write_restore_file(self, name, content):
         if "win" in sys.platform:
@@ -292,13 +304,19 @@ class Q2RadApp(Q2App):
         icons.append("q2gui.ico")
 
         w = q2WaitShow(len(icons))
+        errors = []
         for x in icons:
             w.step(x)
-            self.asset_file_loader(x)
+            if self.asset_file_loader(x) is False:
+                errors.append(x)
         w.close()
         if os.path.isfile("assets/q2gui.ico"):
             shutil.copyfile("assets/q2gui.ico", "assets/q2rad.ico")
 
+        if errors:
+            q2Mess(_("<b>Loading failed for</b>:<br>") + "<br>".join(errors))
+            return
+
         self.set_icon("assets/q2rad.ico")
 
         if os.path.isfile("poetry.lock"):
@@ -308,11 +326,7 @@ class Q2RadApp(Q2App):
         self.write_restore_file(
             "update_q2rad",
             ("" if "win" in sys.platform else "#!/bin/bash\n")
-            + (
-                "q2rad\\scripts\\activate "
-                if "win" in sys.platform
-                else "source q2rad/bin/activate\n"
-            )
+            + ("q2rad\\scripts\\activate " if "win" in sys.platform else "source q2rad/bin/activate")
             + "&& pip install --upgrade q2gui"
             + "&& pip install --upgrade q2db"
             + "&& pip install --upgrade q2report"
@@ -324,9 +338,7 @@ class Q2RadApp(Q2App):
             "run_q2rad",
             ("" if "win" in sys.platform else "#!/bin/bash\n")
             + (
-                "start q2rad\\scripts\\pythonw.exe -m q2rad"
-                if "win" in sys.platform
-                else "q2rad/bin/q2rad\n"
+                "start q2rad\\scripts\\pythonw.exe -m q2rad" if "win" in sys.platform else "q2rad/bin/q2rad\n"
             ),
         )
         if "win" in sys.platform:
@@ -351,9 +363,11 @@ class Q2RadApp(Q2App):
             self.make_desktop_shortcut()
 
     def get_package_versions(self, package):
-        latest_version = json.load(
-            open_url(f"https://pypi.python.org/pypi/{package}/json")
-        )["info"]["version"]
+        response = open_url(f"https://pypi.python.org/pypi/{package}/json")
+        if response:
+            latest_version = json.load(response)["info"]["version"]
+        else:
+            latest_version = None
         current_version = sys.modules[package].__version__
         return latest_version, current_version
 
@@ -364,7 +378,8 @@ class Q2RadApp(Q2App):
             if w.step(package):
                 break
             latest_version, current_version = self.get_package_versions(package)
-            if latest_version != current_version:
+            if latest_version != current_version and latest_version:
+
                 runpip = lambda: subprocess.check_call(  # noqa:E731
                     [
                         sys.executable.replace("w.exe", ".exe"),
@@ -377,6 +392,7 @@ class Q2RadApp(Q2App):
                     ],
                     shell=True if "win" in sys.platform else False,
                 )
+
                 try:
                     runpip()
                 except Exception:
@@ -386,16 +402,10 @@ class Q2RadApp(Q2App):
                         pass
 
                 latest_version, new_current_version = self.get_package_versions(package)
-                upgraded.append(
-                    f"{package} - "
-                    f"<b>{current_version}</b> => "
-                    f"<b>{latest_version}</b>"
-                )
+                upgraded.append(f"{package} - " f"<b>{current_version}</b> => " f"<b>{latest_version}</b>")
         w.close()
         if upgraded:
-            mess = (
-                "Upgrading complete!<p>" "The program will be restarted!" "<p><p>"
-            ) + "<p>".join(upgraded)
+            mess = ("Upgrading complete!<p>" "The program will be restarted!" "<p><p>") + "<p>".join(upgraded)
         else:
             mess = "Updates not found!<p>"
         q2Mess(mess)
@@ -411,12 +421,16 @@ class Q2RadApp(Q2App):
     def check_app_update(self):
         if self.app_url and self.app_version:
             market_version = read_url(self.app_url + ".version").decode("utf-8")
+            print(self.app_url + ".version")
             if market_version != self.app_version:
-                if q2AskYN(
-                    f"Update for App <b>{self.app_title}</b> detected!"
-                    f"<p>Current version <b>{self.app_version}</b>"
-                    f"<p>New version <b>{market_version}</b>"
-                    "<p>Download and install?"
+                if (
+                    q2AskYN(
+                        f"Update for App <b>{self.app_title}</b> detected!"
+                        f"<p>Current version <b>{self.app_version}</b>"
+                        f"<p>New version <b>{market_version}</b>"
+                        "<p>Download and install?"
+                    )
+                    == 2
                 ):
                     data = json.load(open_url(self.app_url + ".json"))
                     AppManager.import_json_app(data)
@@ -464,11 +478,7 @@ class Q2RadApp(Q2App):
             self.db_logic,
         )
         for x in cu.records():
-            menu_path = (
-                x["menu_path"]
-                + "|"
-                + (x["menu_text"] if x["menu_text"] else x["title"])
-            )
+            menu_path = x["menu_path"] + "|" + (x["menu_text"] if x["menu_text"] else x["title"])
 
             def menu_worker(name):
                 def real_worker():
@@ -565,9 +575,7 @@ class Q2RadApp(Q2App):
 
         # add actions
         if form_dic["form_table"]:
-            form_cursor: Q2Cursor = self.db_data.table(
-                table_name=form_dic["form_table"]
-            )
+            form_cursor: Q2Cursor = self.db_data.table(table_name=form_dic["form_table"])
             form_model = Q2CursorModel(form_cursor)
             form.set_model(form_model)
             sql = f"select * from actions where name = '{name}' order by seq"
@@ -582,20 +590,18 @@ class Q2RadApp(Q2App):
                         child_form_name = x["child_form"]
                         form.add_action(
                             x["action_text"],
-                            self.code_runner(x["action_worker"])
-                            if x["action_worker"]
-                            else None,
+                            self.code_runner(x["action_worker"]) if x["action_worker"] else None,
                             child_form=lambda: self.get_form(child_form_name),
                             child_where=x["child_where"],
                             hotkey=x["action_key"],
+                            eof_disabled=1,
                         )
                     else:
                         form.add_action(
                             x["action_text"],
-                            self.code_runner(x["action_worker"], form=form)
-                            if x["action_worker"]
-                            else None,
+                            self.code_runner(x["action_worker"], form=form) if x["action_worker"] else None,
                             hotkey=x["action_key"],
+                            eof_disabled=x["eof_disabled"],
                         )
         return form
 
@@ -603,7 +609,34 @@ class Q2RadApp(Q2App):
         return traceback.format_exc().replace("\n", "<br>").replace(" ", "&nbsp;")
 
     def code_compiler(self, script):
-        script = script.replace("return", "pass;")
+        if "return" in script:
+
+            def count_indent_spaces(line):
+                spaces_count = 0
+                for x in line:
+                    if x == " ":
+                        spaces_count += 1
+                    else:
+                        break
+                return spaces_count
+
+            def_offset_list = []
+            nsl = []
+            for x in script.split("\n"):
+                if "def" in x:
+                    def_offset_list.append(count_indent_spaces(x))
+                elif def_offset_list:
+                    if count_indent_spaces(x) <= def_offset_list[-1]:
+                        def_offset_list.pop()
+                elif re.match(r".*return\W*.*", x):
+                    if "return " in x:
+                        if x.strip() == "return":
+                            x = x.replace("return", "raise ReturnEvent")
+                        else:
+                            x = x.replace("return", "RETURN = ") + "; raise ReturnEvent"
+                nsl.append(x)
+            script = "\n".join(nsl)
+
         try:
             code = compile(script, "'<worker>'", "exec")
             return {"code": code, "error": ""}
@@ -617,19 +650,10 @@ class Q2RadApp(Q2App):
     def code_runner(self, script, form=None, __name__="__main__"):
         _form = form
         # to provide return ability for exec
-        if "return " in script:
-            nsl = []
-            for x in script.split("\n"):
-                if "return " in x:
-                    if x.strip() == "return":
-                        x = x.replace("return", "raise ReturnEvent")
-                    else:
-                        x = x.replace("return", "RETURN = ") + "; raise ReturnEvent"
-                nsl.append(x)
-            script = "\n".join(nsl)
 
         def real_runner():
             # make exec stop on return
+
             class ReturnEvent(Exception):
                 pass
 
@@ -662,12 +686,13 @@ class Q2RadApp(Q2App):
     def run_module(self, name=""):
         script = self.db_logic.get("modules", f"name = '{name}'", "script")
         if script:
-            return self.code_runner(script)()
+            ret = self.code_runner(script)()
+            return ret
 
 
 def main():
     app = Q2RadApp("q2RAD")
-    app.dev_mode = 1
+    app.dev_mode = 0
     app.run()
 
 
