@@ -40,7 +40,7 @@ class Q2RadReport(Q2Report):
     def prepare_output_file(self, output_file):
         rez_name = ""
         if "." not in output_file:
-            rez_name = f"tmp/repo.{output_file}"
+            rez_name = f"temp/repo.{output_file}"
         else:
             form = Q2Form("Report to")
             form.heap.mode = ""
@@ -83,7 +83,7 @@ class Q2RadReport(Q2Report):
             form.do_not_save_geometry = 1
             form.run()
             if form.heap.mode:
-                rez_name = f"tmp/repo.{form.heap.mode}"
+                rez_name = f"temp/repo.{form.heap.mode}"
             else:
                 rez_name = ""
         if rez_name:
@@ -116,9 +116,9 @@ class Q2RadReport(Q2Report):
         super().data_stop()
         self.waitbar.close()
         self.last_focus_widget.set_focus()
-        # print(self.last_focus_widget)
+        q2app.q2_app.process_events()
 
-    def run(self, output_file="tmp/repo.html"):
+    def run(self, output_file="temp/repo.html"):
         output_file = self.prepare_output_file(output_file)
         if not output_file:
             return
@@ -218,6 +218,9 @@ class Q2Reports(Q2Form):
             self.model.update({"name": self.r.name, "content": form.s.json})
             self.set_grid_index(self.current_row)
 
+    def before_form_show(self):
+        self.maximized = True
+
     def after_form_show(self):
         self.anchor: Q2Widget = self.w.anchor
         if self.anchor is not None:
